from __future__ import print_function
import numpy
import pandas as pd
import os

import tensorflow as tf
from tensorflow import keras

filepath="/notebooks/titanic3.xls"
all_df = pd.read_excel("/notebooks/titanic3.xls")
# read xls that order by this cols definition
cols=['survived','name','pclass' ,'sex', 'age', 'sibsp',
      'parch', 'fare', 'embarked']
all_df = all_df[cols]

# generate a boolean array, msk = [False  True False ...  True  True  True], 60% TRUE and 40% FALSE.
msk = numpy.random.rand(len(all_df)) < 0.7
# separator train/test dataframe
train_df = all_df[msk]
test_df = all_df[~msk]

print('total:',len(all_df),
      'train:',len(train_df),
      'test:',len(test_df))

print("_" * 65)
print("* train dataframe:")
print(train_df[:2])

def PreprocessData(raw_df):
    # axis=1(columns), drop name column
    df = raw_df.drop(['name'], axis=1)

    # fill NA with average
    age_mean = df['age'].mean()
    df['age'] = df['age'].fillna(age_mean)
    fare_mean = df['fare'].mean()
    df['fare'] = df['fare'].fillna(fare_mean)

    # transfer feamale to 0 and male to 1
    df['sex']= df['sex'].map({'female':0, 'male': 1}).astype(int)

    # Convert categorical variable into dummy/indicator variables, the result will be "df embarked_C embarked_Q embarked_S"
    # x_OneHot_df is a data frame
    x_OneHot_df = pd.get_dummies(data=df,columns=["embarked" ])

    # get value without title header
    # ndarray is a numpy
    ndarray = x_OneHot_df.values
    # feature is the columns of ndarray other then first columns, [:,1:] => row=all, column>=1
    Features = ndarray[:,1:]
    # label is the first columns of ndarray, [:,0] => row=all, column=0
    Label = ndarray[:,0]

    # normalize feature
    mean = Features.mean(axis=0)
    std = Features.std(axis=0)
    scaledFeatures = (Features - mean) / std

    return scaledFeatures,Label


train_Features,train_Label = PreprocessData(train_df)
test_Features,test_Label = PreprocessData(test_df)

print("_" * 65)
print("* normalized train feature:")
print(train_Features[:2])
print("train label:")
print(train_Label[:2])

model = keras.Sequential()

model.add(keras.layers.Dense(64, input_dim=9, kernel_regularizer=keras.regularizers.l2(0.001), activation='relu'))
model.add(keras.layers.Dropout(0.5))
model.add(keras.layers.Dense(64, kernel_regularizer=keras.regularizers.l2(0.001), activation='relu'))
model.add(keras.layers.Dropout(0.5))
model.add(keras.layers.Dense(1, activation='sigmoid'))

model.compile(loss='binary_crossentropy', 
              optimizer='adam', metrics=['accuracy'])

model.summary()

# Display training progress by printing a single dot for each completed epoch
class PrintDot(keras.callbacks.Callback):
  def on_epoch_end(self, epoch, logs):
    if epoch % 100 == 0: print('')
    print('.', end='')

# The patience parameter is the amount of epochs to check for improvement
early_stop = keras.callbacks.EarlyStopping(monitor='val_loss', patience=20)

# validation_split: Float between 0 and 1. Fraction of the training data to be used as validation data. 
train_history = model.fit(x=train_Features, 
                          y=train_Label, 
                          validation_split=0.1, 
                          epochs=1000, 
                          batch_size=30,
                          verbose=0,
                          callbacks=[early_stop, PrintDot()])

# save all (model and weight) to file in HDF5 format
model.save('my_model.h5')

########### After training
import matplotlib.pyplot as plt
def show_train_history(train_history,train,validation):
    plt.figure()
    plt.plot(train_history.history[train])
    plt.plot(train_history.history[validation])
    plt.title('Train History')
    plt.ylabel(train)
    plt.xlabel('Epoch')
    plt.legend(['train', 'validation'])
    plt.show()

show_train_history(train_history,'acc','val_acc')
show_train_history(train_history,'loss','val_loss')

# validation
scores = model.evaluate(x=test_Features, 
                        y=test_Label)
# print source
print("_" * 65)
print("* evaluate source: ")
i = 0
for name in model.metrics_names:
  print("%s=%f" % (name, scores[i]))
  i+=1

########### Predict
# generate Jack & Rose dataframe
Jack = pd.Series([0 ,'Jack',3, 'male', 23, 1, 0,  5.0000,'S'])
Rose = pd.Series([1 ,'Rose',1, 'female', 20, 1, 0, 100.0000,'S'])

JR_df = pd.DataFrame([list(Jack),list(Rose)],  
                  columns=['survived', 'name','pclass', 'sex', 
                   'age', 'sibsp','parch', 'fare','embarked'])
# add Jack & Rose to dataframe list
all_df = pd.concat([all_df, JR_df])
#print(all_df[-2:])
# generate preprocess data with all dataframe, because JR_df need to normalize
all_Features,Label = PreprocessData(all_df)
probability = model.predict(all_Features[-2:])
print("_" * 65)
print("Jack & Rose feature:")
print(all_Features[-2:])

pd = JR_df
pd.insert(len(all_df.columns), 'probability', probability)
print("_" * 65)
print("* predict:")
print(pd[-2:])
